/*!
 * @file ltp_base.h
 * @brief This is the set of base utility stuff for LTP
 * @version $Revision: 1.20 $ $Author: stephen $
 * @version Copyright (C) 2006 Trinity College Dublin
 * This is pretty much a copy of stuff I need from
 * perfume.h
 */

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Trinity College Dublin (TCD) Licklider
 * transmission protocol (LTP) implementation.
 *
 * The Initial Developer of the Original Code is Trinity College Dublin.
 * Portions created by the Initial Developer are Copyright (C) 2006. All
 * Rights Reserved.
 *
 * Contributor(s): Stephen Farrell <stephen.farrell@cs.tcd.ie>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef LTP_BASE_H
#define LTP_BASE_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

// Time type. The double is because that's what my fav. 
// simulator needs.

#include "../config.h"

/// use >1 thread for listeners etc. (only reason to turn off is gdb simplicity)
#define THREADS

#ifdef LTP_TARGET_CYGWIN
typedef double ltp_time_t;
#endif

#ifdef LTP_TARGET_LINUX
typedef double ltp_time_t;
#endif

#ifdef LTP_TARGET_ARM_LINUX
typedef double ltp_time_t;
#endif

#define LTP_OK 0
#define LTP_ERR 1
#define LTP_BAD_PARAM 2
#define LTP_BAD_INPUT 2
#define LTP_MEMORY 3
#define LTP_FILE 4
#define LTP_RECURS 5
#define LTP_BAD_STATE 6
#define LTP_BAD_VALUE 7
#define LTP_SYSTIME 8
#define LTP_SOCKET 9
#define LTP_UNKNOWN_EXT 100

#ifndef MAXPATH
#define MAXPATH 1024
#endif

#ifndef HEXDIGIT
#define HEXDIGIT(foo) ((foo)<0?'.':((foo)<10?'0'+(foo):'A'-10+(foo)))
#endif

#ifndef MINDEFINED
#define MINDEFINED
#define min(a,b) (((a) < (b)) ? (a) : (b))
#define max(a,b) (((a) > (b)) ? (a) : (b))
#endif

#define LTPSTR_LEN MAXPATH
#define PATHSAFETY 100 ///< This is the amount shorter than MAXPATH that all relevant inputs have to be

/// A string buffer we use when logging - one per source file - why not?
static char ltpstr[LTPSTR_LEN+1];

/// Logging defaults
// #define LTP_DEFAULT_LOG_LEVEL LTP_LOG_ADVISORY ///< Default is advisory logging
#define LTP_DEFAULT_LOG_LEVEL LTP_LOG_MAX ///< Default is all logging
#define LTP_DEFAULT_LOG_FILE_NAME "ltp.log" ///< Default Log file name

/// Log levels - some are synonyms 'cause I forget the exact string often
#define LTP_LOG_0           0 ///< The most important things to log
#define LTP_LOG_ERR         0 ///< The most important things to log
#define LTP_LOG_ERROR       0 ///< The most important things to log
#define LTP_LOG_CRITICAL    0 ///< The most important things to log
#define LTP_LOG_SYSTEM      0 ///< The most important things to log
#define LTP_LOG_SYS         0 ///< The most important things to log
#define LTP_LOG_1           1 ///< Stuff you really ought to remember
#define LTP_LOG_ADVISORY    1 ///< Stuff you really ought to remember
#define LTP_LOG_ADV         1 ///< Stuff you really ought to remember
#define LTP_LOG_2           2 ///< Things (like normal operation) you usually don't want
#define LTP_LOG_OPTIONAL    2 ///< Things (like normal operation) you usually don't want
#define LTP_LOG_3           3 ///< Log everthing
#define LTP_LOG_MAX         3 ///< Log everthing
#define LTP_LOG_ALL         3 ///< Log everthing

/// The number of seconds between STARTTIMETRACEBLOCK events
#define SECONDSPERTRACE 5

/// Ok this one is to preceed a block of code that ought to be run (twice) every 5 seconds
/// (Don't ask about the twice - there is a reason!) ENDTIMEDTRACEBLOCK goes at the other
/// end. The extracondition is something that ought also be true before the code's run, the
/// Obviously you can only really do this once in a single code scope unless you use a
/// different "append" (that's added to the name of the static vars required)
/// If LTP_VERBOSE isn't defined this becomes if(false) and the code's never run
/// If LTP_VERBOSE and LTP_EXTRA_EXTRA_VERBOSE are both defined then we want
/// if(true) which is the same as nothing at all.
#ifndef LTP_VERBOSE
#define STARTTIMEDTRACEBLOCK(append,extracondition) if (0) {
#else
#ifdef LTP_EXTRA_EXTRA_VERBOSE
#define STARTTIMEDTRACEBLOCK(append,extracondition) if (1) {
#else
#define STARTTIMEDTRACEBLOCK(append,extracondition) \
	{ \
	static int append##totaliterations=0; \
	append##totaliterations++;\
	static int append##lastthenll=0;\
	static int append##tracethistime=0;\
	int thenll=(long long)floor(ltp_now());\
	if (	(extracondition)  					\
			&&  								\
			(!append##lastthenll 	 					\
			||									\
			(									\
			(thenll%SECONDSPERTRACE==0) 		\
			&&									\
			(thenll!=append##lastthenll)				\
			)									\
			)								 	\
		){
#endif
#endif

/// This is the other end of the STARTTIMETRACEBLOCK - it just updates control vars.
#ifndef LTP_VERBOSE
#define ENDTIMEDTRACEBLOCK(append) }
#else
#ifdef LTP_EXTRA_EXTRA_VERBOSE
#define  ENDTIMEDTRACEBLOCK(append) }
#else
#define ENDTIMEDTRACEBLOCK(append) \
		if (append##tracethistime==1) {append##lastthenll=thenll;append##tracethistime=0;} \
		else append##tracethistime++; \
	} \
	}
#endif
#endif

// stephen@tolerantnetworks.com: multicast stuff for MUDSAT
#define LTP_MULTICAST
#define LTP_MC_DEBUG


extern const char *ADDRANY; ///< this is an address which is a wildcard (used in whos_listening etc)

/*! 
 * @fn ltp_now(void)
 * @brief return the current time in appropriate format
 * @return is the current time as a ltp_time_t
 */
ltp_time_t ltp_now(void);

/*!
 * @brief close logfile etc. 
 * @return zero or error
 */
int ltp_log_close(void);

/*!
 * @brief re-open logfile etc. (after close, but same params otherwise)
 * @return zero or error
 */
int ltp_log_reopen(void);

/*!
 * @brief initialise (or switch) logging to the named file at the given level
 * @param logfile (platform dependent) pathname of the logfile
 * @param level the threshold level (only this level and lower are actually logged)
 * @return zero for success LTP_OK or error value
 */
extern int ltp_log_init(char *logfile, int level);

/*!
 * @brief initialise logging to the named file
 * @param level is the loglevel of this event
 * @param logstr is the event string
 * @return zero for success LTP_OK or error value
 */
extern int ltp_log(int level, char *logstr);

/*!
 * @brief flush events to the current logfile
 * @return zero for success LTP_OK or error value
 */
extern int ltp_log_flush(void);

/// Logging & error macros 
#define LTPERR(xxFoo) { \
	char xxBar[250]; \
	snprintf(xxBar,250,"Error exit(%d) at %d in %s\n",xxFoo,__LINE__,__FILE__); \
	ltp_log(LTP_LOG_CRITICAL,xxBar); \
	return(xxFoo); \
}
#define VOIDLTPERR(xxFoo) { \
	char xxBar[250]; \
	snprintf(xxBar,250,"Error exit(void - would have been %d) at %d in %s\n",xxFoo,__LINE__,__FILE__); \
	ltp_log(LTP_LOG_CRITICAL,xxBar); \
	return; \
}
#define NULLLTPERR(xxFoo) { \
	char xxBar[250]; \
	snprintf(xxBar,250,"Error exit(void - would have been %d) at %d in %s\n",xxFoo,__LINE__,__FILE__); \
	ltp_log(LTP_LOG_CRITICAL,xxBar); \
	return NULL; \
}
#define LTP_NERRS   9 ///< API internal use only (bounds checks)
#define ISLTPERR(xxVal) ( xxVal < LTP_NERRS && xxVal > LTP_OK)

/// a debugging macro
#ifdef LTP_VERBOSE
#define LTPGOTHERE \
	{ \
		snprintf(perfstr,LTPSTR_LEN,"At line %d\n",__LINE__); \
		ltp_log(LTP_LOG_MAX,perfstr); \
		ltp_log_flush(); \
	}
#else
#define LTPGOTHERE
#endif

#define LTP_FILETYPE_LTP  0
// some default configuration settings
#ifdef LTP_TARGET_LINUX
#define LTP_CFG_DEFAULT_DIR         "ltp/cfg/"      ///< where to put LTP files by default (below home dir)
#endif

#ifdef LTP_TARGET_CYGWIN
#define LTP_CFG_DEFAULT_DIR         "/var/ltp/"     ///< where to put LTP files by default
#endif

#ifdef LTP_TARGET_ARM_LINUX
#define LTP_CFG_DEFAULT_DIR         "/disk/ltp/"    ///< where to put LTP files by default
#endif
#define LTP_FNAME "current.ltp" ///< file name for LTP current state



/*!
 * @brief Write a timestamped message to the logfile (at LTP_LOG_CRITICAL level logging)
 * @param msg is the message to include with the time
 * @return is zero or otherwise for errors
 */
extern int ltp_log_timestamp(const char *msg);

/*!
 * @brief get a handle on the logfile (file) pointer
 * @return is the "FILE *" for the logfile pointer (or NULL if none)
 */
FILE *ltp_logfile_fp(void);

/*!
 * @brief Dump an ascii hex version of a buffer into the log file
 * @param str is a helpful string
 * @param len is the length of the buffer
 * @param buf is the buffer
 *
 * This is always done at the LTP_LOG_MAX level
 */
extern int ltp_log_buffer(char *str, int len, unsigned char *buf);

/*!
 * @brief initialise the random stream for this thread
 * @param idthing is something unique to the stream (e.g. thread fnc addr)
 * @return zero or error
 */
int ltp_rand_init(int idthing);

/*!
 * @brief return a random int
 * @return is something random-ish
 */
int ltp_rand(void);

/*!
 * @brief return a random octet string of the length required
 * @param len is the length in bytes of the required random value
 * @param val is a caller-allocated buffer to hold the random bytes
 * @return zero or error
 */
int ltp_rand_octets(int len, unsigned char *val);

/*!
 * @brief a nicer print form of ltp_time_t
 * @param thetime is a prefume_time_t value
 * @return is a static string
 */
char *ltp_time_str(ltp_time_t thetime);

/*!
 * @brief return the number of seconds since we started
 * @return is the above
 */
int ltp_elapsed(void);

/*!
 * @brief map from YYYYMMDD etc. string to ltp_time_t
 * @param str is the above type string
 * @param timep is the pointer to the ltp_time_t for the result
 * @return zero or error
 */
int str2ltp_time_t(char *str, ltp_time_t	*timep);

/*!
 * @brief case-insensitive str comp
 * @param a first string
 * @param b 2nd string
 * @return non-zero if strings are considered the same; zero otherwise
 */
int ltp_same(char *a, char *b);

/*!
 * @brief guess
 * @param foo bar
 * @return stuff
 */
int ltp_iswhitespace(char foo);

/*!
 * @brief print the time now as a string incl. milliseconds
 * @return ptr to static char * with the string 
 */
char *ltp_now_str(void);

#endif
