/*!
 * @file ltp_file.h
 * @brief This is the internal header for our transport protocol's state to/from disk functions
 * @version $Revision: 1.1.1.1 $ $Author: stephen $
 * @version Copyright (C) 2006 Trinity College Dublin
 * This is based upon draft-irtf-dtnrg-ltp-0?.txt,
 * initially on the file called draft-irtf-dtnrg-ltp-08.txt
 * as posted to the DTNRG list on 20040317 (happy
 * paddy's day!)
 */

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Trinity College Dublin (TCD) Licklider
 * transmission protocol (LTP) implementation.
 *
 * The Initial Developer of the Original Code is Trinity College Dublin.
 * Portions created by the Initial Developer are Copyright (C) 2006. All
 * Rights Reserved.
 *
 * Contributor(s): Stephen Farrell <stephen.farrell@cs.tcd.ie>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef LTP_FILE_H
#define LTP_FILE_H

// Disk handling errors
#define LTPD_OK				LTP_OK 			///< Disk fuction is ok
#define LTPD_ERR			201					///< Some unspecified disk error
#define LTPD_ERROR			201					///< Sime unspecified disk error
#define LTPD_SPACE			202					///< No spac available

/*!
 * @brief write the ltp_state_var to disk
 * @return zero or error
 */
int ltp_sync_2disk(void);

/*!
 * @brief read the disk to ltp_state_var
 * @return zero or error
 */
int ltp_sync_fromdisk(void);

/*!
 * @brief setup disk settings
 * @param dir is the directory to use
 * @return zero or error
 */
int ltp_sync_setup(char *dir);

/*!
 * @brief force disk settings (mainly for testing)
 * @param fname is the exact file to use
 * @return zero or error
 */
int ltp_sync_setup_file(char *file);

#endif
