/*!
 * @file ltp_udp.h
 * @brief These are functions that our l2 needs to do UDP
 * @version $Revision: 1.6 $ $Author: stephen $
 * @version Copyright (C) 2006 Trinity College Dublin
 *
 * Main idea is just to keep the socket stuff together here
 */

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Trinity College Dublin (TCD) Licklider
 * transmission protocol (LTP) implementation.
 *
 * The Initial Developer of the Original Code is Trinity College Dublin.
 * Portions created by the Initial Developer are Copyright (C) 2006. All
 * Rights Reserved.
 *
 * Contributor(s): Stephen Farrell <stephen.farrell@cs.tcd.ie>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef LTP_UDP_H
#define LTP_UDP_H

#ifndef UDP_DEF_PORT
#warning "Having to define UDP_DEF_PORT as 1113 again"
#define UDP_DEF_PORT 1113
#endif

/// The official default port on which to listen for LTP packets 
/// see: http://www.iana.org/assignments/port-numbers
#define L2_DEFPORT UDP_DEF_PORT

/*!
 * @fn int l2_s_udp_init(int port, sockaddr_in *mcp)
 * @brief initialise a listener on some port
 * @param port is the place to listen:-)
 * @param mcp is NULL or (should be) a multicast group address
 * @return zero or error
 */
int l2_s_udp_init(int port,sockaddr_in *mcp);

/*!
 * @fn int l2_s_udp_arrival(void)
 * @brief check if some bytes have arrived there
 * @return zero if nothing, -1 if error, positive if something there!
 */
int l2_s_udp_arrival();

/*!
 * @fn int l2_s_udp_get_arrived_buf(ltp_l2_buffer *buf, ltp_l2_id *peer) 
 * @brief get the bytes that've just arrived
 * @param buf is the buffer into which they ought be put
 * @param peer is the guy who sent 'em to us (as far as we can tell)
 * @return zero or error
 *
 * Note that this function allocates nothing!!! You better copy the outputs
 * as soon as you can or they'll be overwritten. Luckily, that's just what
 * we do do in ltp.cc's default callback handler!
 */
int l2_s_udp_get_arrived_buf(ltp_l2_buffer *buf, ltp_l2_id *peer);

/*!
 * @fn int l2_s_udp_close(void)
 * @brief finish up the listener
 * @return zero or error
 */
int l2_s_udp_close(void);

/*!
 * @fn int l2_c_udp_init(void) 
 * @brief initialise a UDP client to be able to send in a moment
 * @param port is the place to send to
 * @return zero or error 
 */
int l2_c_udp_init(void);

/*!
 * @fn int l2_c_udp_send(ltp_l2_buffer *buf, ltp_l2_id *peer) 
 * @brief send some bytes to that guy
 * @param buf are the bytes
 * @param peer is the indended victim of those bytes
 * @return zero or error
 */
int l2_c_udp_send(ltp_l2_buffer *buf, ltp_l2_id *peer);


/*!
 * @fn int l2_c_udp_close(void)
 * @brief finish up the client (nothing to do really)
 * @return zero or error
 */
int l2_c_udp_close(void);

/*!
 * @fn int l2_get_mtu(char *name)
 * @brief return our best guess at the MTU
 * @return is the guessed MTU or zero if we've no good guess!
 */
int l2_get_mtu(char *name);

/*!
 * @brief let the client caller know the current socket's port
 * @return the port
 */
int l2_c_sending_port(void);

/*!
 * @brief return raw socket handles for debugging purposes
 * @param rx is a ptr that will contain the rx socket handle
 * @param tx is a ptr that will contain the rx socket handle
 * @return zero or error
 */
int l2_udp_status(int *rx, int *tx);

#endif
