/*!
 * @file sendt_cues.cc
 * @brief This is the external interface for our event handling API
 * @version $Revision: 1.3 $ $Author: stephen $
 * @version Copyright (C) 2004 Trinity College Dublin
 */

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Trinity College Dublin (TCD) Licklider
 * transmission protocol (LTP) implementation.
 *
 * The Initial Developer of the Original Code is Trinity College Dublin.
 * Portions created by the Initial Developer are Copyright (C) 2006. All
 * Rights Reserved.
 *
 * Contributor(s): Stephen Farrell <stephen.farrell@cs.tcd.ie>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef SENDT_CUES_H
#define SENDT_CUES_H

/*!
 * @brief re-initialise shcedule loading
 * @param fname schedule file name
 * @return zero or error
 *
 * when fname is NULL, then the current setup is re-initialised - files
 * re-read, structures re-inited, etc.
 */
int sendt_init(char *fname);

/*!
 * @brief tells a node which other nodes are listening now (so we can Tx to them if there're packets)
 * @param now is the time I need to know about
 * @param nlisteners on input is the size of the listeners array, and on output is the number of real entries in the array
 * @param listeners is an array (caller allocated) of the LTP addresses of the currently listening nodes
 * @return zero or error
 */
int sendt_whos_listening(ltp_time_t now, int *cnt, char **listp);

/*!
 * @brief returns my LTP address
 * @param me is the caller allocated ltpaddr structure in which to put the address
 * @return zero or error
 *
 * Note that my address can change as a result of rescheduling - so I better check it occasionally.
 * I'd say that per-reboot should be ok.
 */
char *sendt_whoami(void);

/*!
 * @brief tells a node which other nodes ought to be talking now (so we can decrement timers)
 * @param now is the time I need to know about
 * @param ntalkers on input is the size of the listeners array, and on output is the number of real entries in the array
 * @param talkers is an array (caller allocated) of the LTP addresses of the currently chatting nodes
 * @return zero or error
 */
int sendt_whos_talking(ltp_time_t now, int *cnt,char **listp);

/*!
 * @brief figure out an LTT for a node to which I'm about to transmit
 * @param peer is the node's address
 * @param now is the current time
 * @param ltt is (a ptr to) the LTT at that time
 * @return zero or error
 */
int sendt_ltt(ltpaddr *peer, ltp_time_t now, ltp_time_t *ltt);

#endif
